//=============================================================================
// FireAttachmentEXFR:
//=============================================================================
//   An invisible/owned actor attached to players, used for handling timers and
//   damage from fire, as well as for handling fire visual effects, sound, etc.
//   Could also be modified/renamed for updated poison.
//=============================================================================

class FireAttachmentEXFR extends Effects;

var bool bIgnited;
var ANNAPlayer fireBurner;
var int fireBurnTicksLeft;
var float fireBurnDamageTimer;	// Timer until taking burn tick damage again
var float fireSmokeTimer;	// If a player should be "smoking"
var float fireSpawnEffectTimer;	// Timer until spawning the next fire puff
var float fireBodyTemperature;	// "Temperature" before a player ignites
var vector fireSpawnLocation;	// Fire/ember shared spawn location
var int fakeRandomizer;		// Avoids some FRand() calls
var int fakeRandomizerEmber;	// Avoids some FRand() calls
var float lightAndSoundFader;	// On/off fade over 1.0s
var float levelTimeSeconds;	// Since Level.TimeSeconds isn't valid here

replication
{
	reliable if(Role == ROLE_Authority)
		fireBurnTicksLeft, fireSmokeTimer, levelTimeSeconds,
		VisualIgniteBurstEffect;
}

simulated function Tick(float delta_time)
{
	if(Owner != None)
	{
		SetLocation(Owner.Location);
		
		// Client-only, I think
		if(Role < ROLE_Authority)
		{
			//SetLocation(Owner.Location);
			
			// Check if we should light up and emit client-side flames
			if(fireBurnTicksLeft > 0 || fireSmokeTimer > levelTimeSeconds)
			{
				if(fireBurnTicksLeft > 0)
				{
					if(LightBrightness < 255)
					{
						lightAndSoundFader += delta_time;
						LightBrightness = byte(FMin(255.0 * lightAndSoundFader, 255.0) + 0.5);
						SoundVolume = byte(FMin(180.0 * lightAndSoundFader, 180.0) + 0.5);
						if(lightAndSoundFader > 1.0)
						{
							lightAndSoundFader = 0.0;
						}
					}
				}
				
				fireSpawnEffectTimer -= delta_time;
				if(fireSpawnEffectTimer < 0.0)
				{
					if(fireBurnTicksLeft > 0)
						fireSpawnEffectTimer += 0.1;	// Lots of fire if ignited
					else
						fireSpawnEffectTimer += 0.75;	// Only a little fire if not, for cloakers
					
					FireFX();
				}
			}
			
			// Check if we should turn off the light
			if(fireBurnTicksLeft == 0 || ANNAPlayer(Owner).Health <= 0)
			{
				if(LightBrightness > 0)
				{
					lightAndSoundFader += delta_time;
					LightBrightness = byte(FMax(255.0 * (1.0 - lightAndSoundFader), 0.0) + 0.5);
					SoundVolume = byte(FMax(180.0 * (1.0 - lightAndSoundFader), 0.0) + 0.5);
					if(lightAndSoundFader > 1.0)
					{
						lightAndSoundFader = 0.0;
					}
				}
			}
		}
		
		// Server-only, I think
		else
		{
			levelTimeSeconds = Level.TimeSeconds;
			
			if(fireBurnTicksLeft > 0)
			{	
				fireBurnDamageTimer -= delta_time;
				if(fireBurnDamageTimer <= 0.0)
				{
					fireBurnDamageTimer += 1.25;
					BurnMe();
				}
			}
		}
		
	}
	else
	{
		Destroy();
	}
}

function BurnMe()
{
	local ANNAPlayer myself;
	
	myself = ANNAPlayer(Owner);
	if(myself != None)
		myself.CDXTakeDamageFullBody(4, fireBurner, Vect(0,0,0), 'Burned');
	
	fireBurnTicksLeft -= 1;
	if(fireBurnTicksLeft == 0)
	{
		// Deactivate everything
		ResetVariables(false);
		
		// However, we'll still be smoking for a bit
		fireSmokeTimer = levelTimeSeconds + 3.5;
	}
}

function HandleFlamedDamage(int damage, ANNAPlayer instigated_by, float level_time)
{
	if(!bIgnited)
	{
		// Players/cloakers receive some visible fire effects even before igniting
		fireSmokeTimer = level_time + 3.5;
		
		// Flamed damage heats players up
		if(level_time > fireBodyTemperature)
			fireBodyTemperature = level_time + (float(damage) * 0.125);
		else
			fireBodyTemperature += (float(damage) * 0.125);
		
		// This is the point at which a player is ignited
		if(fireBodyTemperature >= level_time + 3.6)
		{
			RefreshBurningStatus(instigated_by);
			Ignite();
		}
	}
	else
	{
		// If we receive 'Flamed' damage when already ignited, then refresh
		RefreshBurningStatus(instigated_by);
	}
}

function RefreshBurningStatus(ANNAPlayer instigated_by)
{
	// Assign most recent burner if the damage source was actually a player
	if(instigated_by != None)
		fireBurner = instigated_by;
		
	// Set remaining burn ticks to max
	fireBurnTicksLeft = 9;
}

simulated function RandomSpawnLocation()
{
	fireSpawnLocation.X = (1 - (2 * FRand())) * 9.0;
	fireSpawnLocation.Y = (1 - (2 * FRand())) * 9.0;
	fireSpawnLocation.Z = ((1 - (2 * FRand())) * 14.0) + 15.0;
	fireSpawnLocation += Owner.Location;
}

simulated function FireFX()
{
	local FirePuffEXFR fire;
	
	RandomSpawnLocation();
	
	// One fire puff
	fire = Spawn(class'FirePuffEXFR',,, fireSpawnLocation);
	fire.RemoteRole = ROLE_None;
	fire.SetOwner(Owner);
	fire.InitializeMe(0);
	AssignFireTexture(fire);
	
	// One ember
	fire = Spawn(class'FirePuffEXFR',,, fireSpawnLocation);
	fire.RemoteRole = ROLE_None;
	fire.SetOwner(Owner);
	fire.InitializeMe((fakeRandomizerEmber++ % 4) + 1);
	AssignFireTexture(fire);
}

simulated function VisualIgniteBurstEffect(ANNAPlayer victim)
{
	local FireBurstEXFR burst;
	local int i;
	
	// Client-only, I think
	if(Role < ROLE_Authority)
	{
		// Combustion effect
		for(i = -8; i < 8; ++i)
		{
			burst = Spawn(Class'FireBurstEXFR');
			burst.RemoteRole = ROLE_None;
			burst.SetOwner(victim);
			burst.InitializeMe(i);
			AssignFireTexture(burst);
		}
	}
}

function Ignite()
{
	local ANNAPlayer myself;
	local Pawn p;
	
	bIgnited = true;
	fireBurnDamageTimer = 1.25;
	
	AmbientSound = Sound'Ambient.Ambient.FireMedium1';
	
	myself = ANNAPlayer(Owner);
	if(myself != None)
	{
		// Notify player if they're getting burned for the first time
		myself.ServerConditionalNotifyMsg(5);
		
		// Take some burst damage upon igniting
		myself.CDXTakeDamageFullBody(16, fireBurner, Vect(0,0,0), 'Burned');
		
		// Play some distinct sounds to alert the victim and attacker of the damage occurring
		myself.PlaySound(sound'BasketballSwoosh', SLOT_Misc, 5.0,,, 0.35);
		myself.PlaySound(sound'MaleEyePain', SLOT_Pain, FMax(myself.TransientSoundVolume, 2.0),,, myself.RandomPitch());
		myself.LastPainSound = levelTimeSeconds;
	}
	
	// Server instructs each player to perform a client-side visual effect
	for(p = Level.PawnList; p != None; p = p.nextPawn)
	{
		if(p.IsA('ANNAPlayer'))
		{
			if(ANNAPlayer(p).fireAttach != None)
			{
				ANNAPlayer(p).fireAttach.VisualIgniteBurstEffect(ANNAPlayer(Owner));
			}
		}
	}
}

simulated function PreBeginPlay()
{
	Super.PreBeginPlay();
	ResetVariables(true);
}

function ResetVariables(bool emergency_kill)
{
	bIgnited = false;
	fireBurner = None;
	fireBurnTicksLeft = 0;
	fireBurnDamageTimer = 0.0;
	fireSmokeTimer = 0.0;
	fireSpawnEffectTimer = 0.0;
	fireBodyTemperature = 0.0;
	fakeRandomizer = 0;
	fakeRandomizerEmber = 0;
	lightAndSoundFader = 0.0;
	
	if(emergency_kill)
	{
		AmbientSound = None;
		LightBrightness = 0;
	}
}

simulated function AssignFireTexture(Actor fiery_thing)
{
	switch(fakeRandomizer % 16)
	{
		case 0:
			fiery_thing.OddsOfAppearing = 2.0;
			fiery_thing.Texture = Texture'DeusExItems.Skins.FlatFXTex11';
			break;
		case 1:
			fiery_thing.OddsOfAppearing = 2.0;
			fiery_thing.Texture = Texture'DeusExItems.Skins.FlatFXTex12';
			break;
		case 2:
			fiery_thing.OddsOfAppearing = 2.0;
			fiery_thing.Texture = Texture'DeusExItems.Skins.FlatFXTex13';
			break;
		case 3:
			fiery_thing.OddsOfAppearing = 2.0;
			fiery_thing.Texture = Texture'DeusExItems.Skins.FlatFXTex14';
			break;
		case 4:
			fiery_thing.OddsOfAppearing = 2.0;
			fiery_thing.Texture = Texture'DeusExItems.Skins.FlatFXTex15';
			break;
		case 5:
			fiery_thing.OddsOfAppearing = 2.0;
			fiery_thing.Texture = Texture'DeusExItems.Skins.FlatFXTex16';
			break;
		case 6:
			fiery_thing.OddsOfAppearing = 2.0;
			fiery_thing.Texture = Texture'DeusExItems.Skins.FlatFXTex17';
			break;
		case 7:
			fiery_thing.OddsOfAppearing = 2.0;
			fiery_thing.Texture = Texture'DeusExItems.Skins.FlatFXTex18';
			break;
		case 8:
			fiery_thing.OddsOfAppearing = 1.0;
			fiery_thing.Texture = Texture'DeusExItems.Skins.FlatFXTex20';
			break;
		case 9:
			fiery_thing.OddsOfAppearing = 1.0;
			fiery_thing.Texture = Texture'DeusExItems.Skins.FlatFXTex21';
			break;
		case 10:
			fiery_thing.OddsOfAppearing = 1.0;
			fiery_thing.Texture = Texture'DeusExItems.Skins.FlatFXTex22';
			break;
		case 11:
			fiery_thing.OddsOfAppearing = 1.0;
			fiery_thing.Texture = Texture'DeusExItems.Skins.FlatFXTex23';
			break;
		case 12:
			fiery_thing.OddsOfAppearing = 1.0;
			fiery_thing.Texture = Texture'DeusExItems.Skins.FlatFXTex24';
			break;
		case 13:
			fiery_thing.OddsOfAppearing = 1.0;
			fiery_thing.Texture = Texture'DeusExItems.Skins.FlatFXTex25';
			break;
		case 14:
			fiery_thing.OddsOfAppearing = 1.0;
			fiery_thing.Texture = Texture'DeusExItems.Skins.FlatFXTex26';
			break;
		case 15:
			fiery_thing.OddsOfAppearing = 1.0;
			fiery_thing.Texture = Texture'DeusExItems.Skins.FlatFXTex27';
			break;
	}
	fakeRandomizer += 7;
}

defaultproperties
{
	bHidden=True
	SoundPitch=52
	LightType=LT_Steady
	LightEffect=LE_NonIncidence
	LightHue=19
	LightRadius=4
}