//=============================================================================
// FireballEXFR:
//=============================================================================
//   Fireball for experimental flamethrower.
//=============================================================================

class FireballEXFR extends DeusExProjectile;

#exec OBJ LOAD FILE=Effects
#exec OBJ LOAD FILE=Ambient

var bool bExtinguishing;		// Fading out harmlessly
var bool bBouncedOnce;			// Can damage self after flame bounces
var FireballHeatEXFR heatActor;		// Heat AOE
var float heatActorUpdateTimer;
var float heatRadius;
var byte heatDamageVictimsCounter;
var byte heatDamageVictimsList[5];

simulated singular function Touch(Actor other_actor)
{
	// Don't check anything unless we're the server
	if(Role < ROLE_Authority)
		return;
	
	// Don't check for any collisions anymore if we're extinguishing
	if(bExtinguishing)
		return;
	
	if(other_actor.IsA('Pawn')) //added by Cozmo (for future botplayers etc)
	{
		// Check the most common case first: Players
		if(other_actor.IsA('ANNAPlayer'))
		{
			if(other_actor == Instigator)
			{
				if(bBouncedOnce)
				{
					ApplyHeatDamageToPlayer(ANNAPlayer(other_actor));
					ExtinguishMe();
				}
			}
			else if(TeamDMGame(Level.Game) != None)
			{
				if(ANNAPlayer(other_actor).PlayerReplicationInfo.team != ANNAPlayer(Instigator).PlayerReplicationInfo.team)
				{
					ApplyHeatDamageToPlayer(ANNAPlayer(other_actor));
				}
				ExtinguishMe();
			}
		}
		else
		{
			Pawn(other_actor).TakeDamage(Damage, None, other_actor.Location, Vect(0,0,0), 'Shot');
		}
	}
	
	// Somewhat common case: Decorations
	else if(other_actor.IsA('DeusExDecoration'))
	{
		DeusExDecoration(other_actor).TakeDamage(Damage, None, other_actor.Location, Vect(0,0,0), 'Shot');
		ExtinguishMe();
	}
	
	// Uncommon case: Grenades
	else if(other_actor.IsA('ThrownProjectile'))
	{
		// We can always destroy our own grenades
		if(ThrownProjectile(other_actor).Owner == ANNAPlayer(Instigator))
		{
			ThrownProjectile(other_actor).TakeDamage(Damage, Instigator, other_actor.Location, Vect(0,0,0), 'Shot');
		}
		
		// In team games, only blow up enemy grenades
		else if(TeamDMGame(Level.Game) != None)
		{
			if(ThrownProjectile(other_actor).team != ANNAPlayer(Instigator).PlayerReplicationInfo.team)
			{
				ThrownProjectile(other_actor).TakeDamage(Damage, Instigator, other_actor.Location, Vect(0,0,0), 'Shot');
			}
		}
		
		// This must be deathmatch, blow it up
		else
		{
			ThrownProjectile(other_actor).TakeDamage(Damage, Instigator, other_actor.Location, Vect(0,0,0), 'Shot');
		}
	}
	
	// Rare case: BlockAlls
	else if(other_actor.IsA('BlockAll'))
	{
		ExtinguishMe();	
	}
}

auto simulated state Flying
{
	simulated function Tick(float delta_time)
	{
		local float intensity, f1, f2;
		
		// Server-side damage
		if(Role == ROLE_Authority)
		{
			// Attempt to hurt nearby players with heat
			if(!bExtinguishing)
			{
				heatActorUpdateTimer += delta_time;
				if(heatActorUpdateTimer >= 0.15 && heatActor != None)
				{
					heatActorUpdateTimer -= 0.15;
					HeatActorUpdate();
				}
			}
			
			// Unless we're extinguishing, then just die off harmlessly
			else
			{
				LifeSpan -= delta_time * 2.6;
			}
		}
		
		// Client-side visual FX
		else
		{
			if(bExtinguishing)
			{
				LifeSpan -= delta_time * 2.6;
			}
			
			intensity = LifeSpan / Default.LifeSpan;
			
			// Opacity fading
			f1 = Sin(intensity * Pi * 0.5);
			f2 = Sin((1.0 - intensity) * Pi * 0.5);
			DrawScale = OddsOfAppearing * 1.25 * (1.0 - f1) + 0.2;
			ScaleGlow = 3.0 * (f2 - (1.0 - f1));
		}
	}
	simulated function HitWall(vector hit_normal, actor hit_wall)
	{
		local Vector new_location;
		
		// Update the fireball
		bBouncedOnce = true;
		Velocity = 0.7 * ((Velocity dot hit_normal) * hit_normal * (-2.0) + Velocity);
		new_location = Location + hit_normal;
		SetLocation(new_location);
		
		// Also update heatActor
		if(heatActor != None)
		{
			heatActor.SetLocation(new_location);
			heatActor.Velocity = Velocity;
		}
	}
	simulated function ZoneChange(ZoneInfo new_zone)
	{
		Super.ZoneChange(new_zone);
		
		if(new_zone.bWaterZone)
		{
			ExtinguishMe();
			if(Role < ROLE_Authority)
			{
				// Steam-like sound
				if(FRand() < 0.65)
					PlaySound(Sound'MedicalHiss', SLOT_Misc,,, 1300, 0.7 - 0.2 * FRand());
			}
		}
	}
}

function ExtinguishMe()
{
	if(heatActor != None)
	{
		heatActor.Destroy();
		heatActor = None;
	}
	
	bExtinguishing = true;
	RemoteRole = ROLE_None;
}

function HeatActorUpdate()
{
	heatRadius = ((Default.LifeSpan - LifeSpan) / Default.LifeSpan) * 30.0 + 5.0;
	heatActor.SetCollisionSize(heatRadius, heatRadius);
}

function ApplyHeatDamageToPlayer(ANNAPlayer victim)
{
	local byte i;
	
	// This check is kinda cringe, because really all 'Flamed' damage should be ignored for players in water,
	//  but if this check was in ANNAPlayer instead, WPs would do no damage to underwater targets
	if(victim.Region.Zone.bWaterZone)
		return;
	
	if(heatDamageVictimsCounter >= 5)
		return;
	
	// If a player has already been burned by heat radius damage from THIS fireball, then
	//  don't allow it to damage them again, even upon direct contact
	for(i = 0; i < 5; ++i)
	{
		if(victim.PlayerReplicationInfo.PlayerID == HeatDamageVictimsList[i])
		{
			return;
		}
	}
	
	// Damage the player and add them to the victims list
	victim.CDXTakeDamageFullBody(Damage, Instigator, Vect(0,0,0), 'Flamed');
	HeatDamageVictimsList[HeatDamageVictimsCounter++] = victim.PlayerReplicationInfo.PlayerID;
}

simulated event Destroyed()
{
	if(heatActor != None)
	{
		heatActor.Destroy();
		heatActor = None;
	}
	
	Super.Destroyed();
}

function PostPostBeginPlay()
{
	heatActor.Velocity = Velocity;
	Super.PostPostBeginPlay();
}

function PreBeginPlay()
{
	local byte b;
	
	Super.PreBeginPlay();
	
	// The AOE heat actor gets spawned along with the visual fireball
	heatActor = Spawn(Class'FireballHeatEXFR', Self,, Location);
	
	// Set the texture
	if(ANNAPlayer(Owner).fireAttach != None)
		ANNAPlayer(Owner).fireAttach.AssignFireTexture(self);
	
	// Slightly random radius
	LightRadius = Rand(4) + 1;
	
	// Initialize the PlayerID list to 255, since the default 0 is indeed a valid PlayerID
	for(b = 0; b < 5; ++b)
		HeatDamageVictimsList[b] = 255;
}

defaultproperties
{
	bIgnoresNanoDefense=true
	heatRadius=10.000000
	speed=850.000000
	MaxSpeed=850.000000
	LifeSpan=0.400000
	DrawType=DT_Sprite
	Style=STY_Translucent
	bUnlit=True
	CollisionRadius=5.000000
	CollisionHeight=5.000000
	LightType=LT_Steady
	LightEffect=LE_NonIncidence
	LightBrightness=255
	LightHue=19
}